"""
"""

import json
import logging
import sys

import click

from . import auth
from . import record


logger = logging.getLogger(__name__)


@click.command()
@click.option('-v', '--verbose', is_flag=True, help='Verbose output.')
@click.option('-d', '--debug', is_flag=True, help='Output debugging statements.')
@click.option('-c', '--credentials_file', type=click.File(), help='Credentials file to use.')
@click.option('-o', '--output_file', type=click.File('w'), help='Output file to store record.')
def main(verbose, debug, credentials_file, output_file):
    logging_level = None
    if verbose: logging_level = logging.INFO
    if debug: logging_level = logging.DEBUG

    logging.basicConfig(stream=sys.stdout, level=logging_level)

    credentials = None
    if credentials_file:
        logger.info(f'Reading credentials from "{credentials_file.name}"')
        credentials = json.load(credentials_file)
        username = credentials['username']
        password = credentials['password']
    else:
        username = click.prompt('username')
        password = click.prompt('password', hide_input=True)

    token = auth.get_token(username, password)
    student_record = record.get_student_record(token)

    if output_file:
        logger.info(f'Writing student record to "{output_file.name}"')
        json.dump(student_record, output_file, indent=2, sort_keys=True)
    else:
        click.echo(json.dumps(student_record, indent=2, sort_keys=True))
