"""
"""

import logging
import re

import requests


logger = logging.getLogger(__name__)

authtoken_url = 'https://my.concordia.ca/psp/upprpr9/'
authservice_url = 'https://genesis.concordia.ca/mportal2.0/services/authservice.ashx'


class SessionExpired(Exception):
    pass


class BadAuthtoken(Exception):
    pass


class AuthenticationFailed(Exception):
    pass


def get_token(username, password, verify_token=True):
    authtoken = _api_get_authtoken(username, password)
    token = _api_get_token(authtoken)
    if verify_token:
        user_info = _get_user_info(token)
    return token


def _get_user_info(token):
    logger.info('Getting user info')
    data = {
        'token': token,
        'method': 'who-am-i',
    }
    r = requests.post(authservice_url, data=data)
    resp = r.json()
    if resp['status'] == 'ERR':
        if resp['error']['message'] == 'Session expired':
            raise SessionExpired(resp['error']['message'])
        raise Exception('Unknown error: ' + resp['error']['message'])
    user_info = resp['result']
    logger.info('User_info: ' + str(user_info))
    return user_info


def _api_get_authtoken(username, password):
    logger.info('Getting new authtoken')
    params = {
        'cmd': 'login',
        'languageCd': 'ENG',
    }
    data = {
        'device': 'mobile',
        'cufld': password,
        'resource': '/content/cspace/en/login.html',
        '_charset_': 'UTF-8',
        'userid': username,
        'pwd': password,
    }
    r = requests.post(authtoken_url, params=params, data=data)
    m = re.search('\?tokenauth=([a-zA-Z0-9]{28})&', r.text)
    if not m:
        raise AuthenticationFailed('Authtoken not found in response')
    authtoken = m.group(1)
    logger.info('Authtoken: ' + authtoken)
    return authtoken


def _api_get_token(authtoken):
    logger.info('Getting new token')
    data = {
        'authtoken': authtoken,
        'method': 'tokenauthenticate',
    }
    r = requests.post(authservice_url, data=data)
    resp = r.json()
    if resp['status'] == 'ERR':
        error_message = resp['error']['message'] + ': ' + resp['error']['cause']['message']
        if resp['error']['message'] == 'Authentication failed' and resp['error']['cause']['message'] == 'Unknown user':
            raise BadAuthtoken(error_message)
        raise Exception('Unknown error: ' + error_message)
    token = resp['token']
    logger.info('Token: ' + token)
    return token
